@@ -8,7 +8,6 @@ from enum import Enum, IntEnum
 from base64 import b64encode
 from base64 import b64decode
 
-from typing import Callable
 from typing import List
 from typing import Optional
 from typing import Tuple
@@ -24,13 +23,13 @@ class DecopError(Exception):
     """A generic DeCoP error."""
 
 
-class DecopValueError(Exception):
+class DecopValueError(DecopError):
     """A DeCoP value conversion error."""
     def __init__(self, value: str, expected_type: Optional[type] = None):
         if expected_type:
-            super().__init__("Failed to convert {!r} to type '{}'".format(value, expected_type))
+            super().__init__(f"Failed to convert {value!r} to type '{expected_type}'")
         else:
-            super().__init__("Failed to infer type for {!r}".format(value))
+            super().__init__(f"Failed to infer type for {value!r}")
 
 
 DecopType = Union[bool, int, float, str, bytes, tuple]
@@ -39,8 +38,9 @@ DecopMetaType = Type[DecopType]
 DecopStreamType = Union[str, bytes]
 DecopStreamMetaType = Type[DecopStreamType]
 
-DecopCallback = Callable[[datetime, str, DecopType, Optional[DecopError]], None]
-DecopMonitoringLine = Tuple[datetime, str, Union[str, DecopError]]
+Timestamp = datetime
+
+DecopMonitoringLine = Tuple[Timestamp, str, Union[str, DecopError]]
 
 
 class UserLevel(IntEnum):
@@ -66,6 +66,36 @@ class StreamType(Enum):
     BASE64 = 2
 
 
+class SubscriptionValue:
+    """Wrapper type for either the value of a subscription or any exception that may have been raised.
+
+    Attributes:
+        value (Union[DecopType, DecopError]): The value from the subscription or an exception that may have occurred.
+
+    """
+    def __init__(self, value: Union[DecopType, DecopError]):
+        self._value = value
+
+    def get(self) -> DecopType:
+        """Returns the value from the subscription or re-raises any exception that may have occurred.
+
+        Returns:
+            DecopType: The value from the subscription.
+
+        Raises:
+            DecopError: The exception that has been raised while handling the subscription value.
+
+        """
+        if isinstance(self._value, DecopError):
+            raise self._value
+        return self._value
+
+    @property
+    def value(self) -> Union[DecopType, DecopError]:
+        """Union[DecopType, DecopError]: The value or exception of a subscription."""
+        return self._value
+
+
 def user_level(user_level_str: Optional[str]) -> Optional[UserLevel]:
     """Converts a string with a parameter user level to the corresponding type.
 
@@ -159,7 +189,7 @@ def encode_value(value: DecopType) -> str:
     if isinstance(value, bytes):
         return '"' + b64encode(value).decode() + '"'
 
-    raise DecopError('Invalid type for value: {}'.format(type(value)))
+    raise DecopError(f'Invalid type for value: {type(value)}')
 
 
 def decode_value(result: str, expected_type: DecopMetaType) -> DecopType:
@@ -216,7 +246,7 @@ def decode_value(result: str, expected_type: DecopMetaType) -> DecopType:
             return tuple(result[2:-1].split(' '))
         raise DecopValueError(result, tuple)
 
-    raise DecopError("Unexpected type while decoding a DeCoP value: '{}'".format(expected_type))
+    raise DecopError(f"Unexpected type while decoding a DeCoP value: '{expected_type}'")
 
 
 def decode_value_inferred(param_str: str) -> DecopType:
@@ -428,12 +458,12 @@ def _match_tuple(text: str) -> Tuple[str, Optional[tuple]]:
     if len(text) < 2 or text[0] != '(':
         return text, None
 
-    ls = []  # type: List[DecopType]
+    ls: List[DecopType] = []
     original = text[:]
     text = text[1:]
 
     while len(text) > 0:
-        value = None  # type: Optional[DecopType]
+        value: Optional[DecopType] = None
         if text[0] == '#':
             text, value = _match_bool(text)
         elif text[0] == '"':
@@ -473,22 +503,22 @@ def parse_monitoring_line(line: str) -> DecopMonitoringLine:
     datetime_fmt = '%Y-%m-%dT%H:%M:%S.%fZ'
 
     if line.lower().startswith('(error: '):
-        match = re.match(r"\(Error: (.*?) \((.*?) '(.*?)\) (.*?)\)\r?\n", line, re.IGNORECASE)
+        match = re.match(r"\(Error: (.*?) \((.*?) '(.*?)\) (.*?)\)\r?\n?", line, re.IGNORECASE)
 
         if match is not None:
             ls = match.groups()
             timestamp = datetime.strptime(ls[1], datetime_fmt)
-            error = DecopError('Error: {} {}'.format(ls[0], ls[3]))
+            error = DecopError(f'Error: {ls[0]} {ls[3]}')
             return timestamp, ls[2], error
     else:
-        match = re.match(r"\((.*?) '(.*?) (.*?)\)\r?\n", line)
+        match = re.match(r"\((.*?) '(.*?) (.*?)\)\r?\n?", line)
 
         if match is not None:
             ls = match.groups()
             timestamp = datetime.strptime(ls[0], datetime_fmt)
             return timestamp, ls[1], ls[2]
 
-    error = DecopError('Invalid monitoring line message: {!r}'.format(line))
+    error = DecopError(f'Monitoring line message is invalid: {line!r}')
     return datetime.now(), '', error
 
 
@@ -546,8 +576,8 @@ class Typedef:
     def __init__(self, name: str, is_atomic: bool) -> None:
         self.name = name
         self.is_atomic = is_atomic
-        self.params = OrderedDict()  # type: OrderedDict[str, Parameter]
-        self.cmds = OrderedDict()  # type: OrderedDict[str, Command]
+        self.params: OrderedDict[str, Parameter] = OrderedDict()
+        self.cmds: OrderedDict[str, Command] = OrderedDict()
 
 
 class Module:
@@ -559,8 +589,8 @@ class Module:
     """
     def __init__(self, name: str) -> None:
         self.name = name
-        self.params = OrderedDict()  # type: OrderedDict[str, Parameter]
-        self.cmds = OrderedDict()  # type: OrderedDict[str, Command]
+        self.params: OrderedDict[str, Parameter] = OrderedDict()
+        self.cmds: OrderedDict[str, Command] = OrderedDict()
 
 
 class SystemModel:
@@ -569,8 +599,8 @@ class SystemModel:
     def __init__(self) -> None:
         self._logger = logging.getLogger(__name__)
         self.name = ''
-        self.typedefs = OrderedDict()  # type: OrderedDict[str, Typedef]
-        self.modules = OrderedDict()  # type: OrderedDict[str, Module]
+        self.typedefs: OrderedDict[str, Typedef] = OrderedDict()
+        self.modules: OrderedDict[str, Module] = OrderedDict()
 
     def build_from_file(self, filename: str) -> None:
         """Loads a DeCoP system model from an XML file.
