@@ -50,20 +50,22 @@ def download_system_xml(ip_or_name: str) -> Tuple[str, Optional[str]]:
             host = ipaddress.ip_address(socket.gethostbyname(ip_or_name))
         except (ValueError, OSError):
             net = NetworkConnection('')
-            print("Searching for device '{}'...".format(ip_or_name))
-            host, _, _ = asyncio.get_event_loop().run_until_complete(net.find_device(ip_or_name))
-            if host is None:
-                print("Host not found: '{}'".format(ip_or_name))
+            print(f"Searching for device '{ip_or_name}'...")
+            network_address = asyncio.get_event_loop().run_until_complete(net.find_device(ip_or_name))
+            if network_address is None:
+                print(f"Host not found: '{ip_or_name}'")
                 return '', None
+            host, _, _ = network_address
 
-    url = 'http://{}/system.xml'.format(host)
+    # noinspection HttpUrlsUsage
+    url = f'http://{host}/system.xml'
 
     try:
-        print("Downloading system model from {}...".format(url))
+        print(f"Downloading system model from {url}...")
         with urllib.request.urlopen(url) as response:
             return url, response.read().decode('utf-8')
     except urllib.error.HTTPError as ex:
-        print("Unable to download: '{}'".format(ex))
+        print(f"Unable to download: '{ex}'")
         return url, None
 
 
@@ -93,7 +95,7 @@ def make_parameter(param: Parameter, typenames: TypeNameMap, readlevel: UserLeve
     py_name, py_type = None, None
 
     if is_readable:
-        py_name = make_name(param.name)
+        py_name = make_function_name(param.name)
         if is_scalar_type(param.paramtype):
             py_type = make_type(param.paramtype, is_readable, is_writeable, is_readset)
         else:
@@ -145,10 +147,11 @@ def make_stream_type(stream_type: Optional[StreamType]) -> str:
 
 
 def make_type(name: str, is_readable: bool = True, is_writeable: bool = True, is_readset: bool = False) -> str:
-    """Converts a DeCoP type name to a Python type name.
+    """Converts a DeCoP type name to a Python type name. This can either be a scalar type like 'REAL' or
+       'INTEGER', or a typedef name like 'mc-board' or 'laser.
 
     Args:
-        name (str): The name of the parameter.
+        name (str): The name of the type.
         is_readable (bool): True if the parameter is readable, false otherwise.
         is_writeable (bool): True if the parameter is writeable, false otherwise.
         is_readset (bool): True if the parameter is a readset parameter, false otherwise.
@@ -187,18 +190,70 @@ def make_type(name: str, is_readable: bool = True, is_writeable: bool = True, is
         if is_readable:
             return 'DecopBinary'
 
+    # 'dev_can_1__modules__LNC_19__can_desc' -> 'Dev_Can_1__Modules__Lnc_19__Can_Desc'
     name = name.title()
 
-    for c in '_-#*%.:':
-        name = name.replace(c, '')
+    # 'Dev_Can_1__Modules__Lnc_19__Can_Desc' -> 'DevCan1ModulesLnc19CanDesc'
+    name = name.replace('-', '')  # Minus
+    name = name.replace('_', '')  # Underscore
 
+    if not name.isalnum():
+        print(f"Error: Type '{name}' contains invalid characters")
+
+    # Prevent collisions with Python keywords
     if keyword.iskeyword(name):
+        print(f"Warning: Type '{name}' collides with keyword and will be renamed to '{name}_'")
         return name + '_'
 
     return name
 
 
-def make_name(name: str) -> str:
+def is_builtin_name(name: str) -> bool:
+    """Tests if a name is the same as one of the Python built-in function names.
+
+    Args:
+        name (str): The name to test for.
+
+    Returns:
+        bool: True if the name is a Python built-in function name, False otherwise.
+
+    """
+
+    return name in ['abs', 'aiter', 'all', 'any', 'anext', 'ascii', 'bin', 'bool', 'breakpoint', 'bytearray', 'bytes',
+                    'callable', 'chr', 'classmethod', 'compile', 'complex', 'delattr', 'dict', 'dir', 'divmod',
+                    'enumerate', 'eval', 'exec', 'filter', 'float', 'format', 'frozenset', 'getattr', 'globals',
+                    'hasattr', 'hash', 'help', 'hex', 'id', 'input', 'int', 'isinstance', 'issubclass', 'iter', 'len',
+                    'list', 'locals', 'map', 'max', 'memoryview', 'min', 'next', 'object', 'oct', 'open', 'ord', 'pow',
+                    'print', 'property', 'range', 'repr', 'reversed', 'round', 'set', 'setattr', 'slice', 'sorted',
+                    'staticmethod', 'str', 'sum', 'super', 'tuple', 'type', 'vars', 'zip']
+
+
+def make_function_name(name: str) -> str:
+    """Generates a valid Python function name from a system model name, e.g. 'temp-act' -> 'temp_act'.
+
+    Args:
+        name (str): The system model name, e.g. 'temp-act'.
+
+    Returns:
+        str: The generated function name, e.g. 'temp_act'.
+
+    """
+
+    # 'temp-act' -> 'temp_act'
+    name = name.replace('-', '_')  # Minus -> Underscore
+
+    if not name.replace('_', '').isalnum():
+        print(f"Error: Function name '{name}' contains invalid characters")
+
+    # Prevent collisions with Python keywords
+    if keyword.iskeyword(name):
+        print(f"Warning: Function '{name}' collides with keyword and will be renamed to '{name}_'")
+        return name + '_'
+
+    return name
+
+
+def make_param_name(name: str) -> str:
     """Generates a valid Python name from a system model name.
 
     Args:
@@ -208,16 +263,24 @@ def make_name(name: str) -> str:
         str: The generated name.
 
     """
-    for c in '-#*%.:':
-        name = name.replace(c, '_')
 
+    # 'temp-act' -> 'temp_act'
+    name = name.replace('-', '_')  # Minus -> Underscore
+
+    name = name.lower()
+
+    if not name.replace('_', '').isalnum():
+        print(f"Error: Parameter name '{name}' contains invalid characters")
+
+    # Prevent collisions with Python keywords
     if keyword.iskeyword(name):
+        print(f"Warning: Parameter '{name}' collides with keyword and will be renamed to '{name}_'")
         return name + '_'
 
-    if name == 'id':
-        return 'id_'
-    if name == 'type':
-        return 'type_'
+    # Prevent collisions with Python built-in function names
+    if is_builtin_name(name):
+        print(f"Warning: Parameter '{name}' collides with built-in function and will be renamed to '{name}_'")
+        return name + '_'
 
     return name
 
@@ -308,7 +371,7 @@ def infer_type_names(model: SystemModel, ul: UserLevel) -> TypeNameMap:
         TypeNameMap: A list of type name substitutions.
 
     """
-    user_levels = collections.defaultdict(set)  # type: UserLevelMap
+    user_levels: UserLevelMap = collections.defaultdict(set)
 
     # Infer the used user levels for all used typedef
     for node in model.modules.values():
@@ -317,7 +380,7 @@ def infer_type_names(model: SystemModel, ul: UserLevel) -> TypeNameMap:
                 user_levels[param.paramtype].add((param.readlevel, param.writelevel))
                 infer_type_names_helper(model, user_levels, model.typedefs[param.paramtype], param.readlevel, param.writelevel, ul)
 
-    result = {}  # type: TypeNameMap
+    result: TypeNameMap = {}
 
     # Generate a unique name for each typedef and user level
     for typename, userlevel_list in user_levels.items():
@@ -348,10 +411,10 @@ def generate_change_ul(use_async: bool) -> str:
     """
     result = ""
 
-    result += "    {_async}def change_ul(self, ul: UserLevel, passwd: str) -> int:\n".format(_async='async ' if use_async else '')
-    result += "        assert isinstance(ul, UserLevel), \"expected type 'UserLevel' for parameter 'ul', got '{}'\".format(type(ul))\n"
-    result += "        assert isinstance(passwd, str), \"expected type 'str' for parameter 'passwd', got '{}'\".format(type(passwd))\n"
-    result += "        return {_await}self.__client.change_ul(ul, passwd)".format(_await='await ' if use_async else '')
+    result += f"    {'async ' if use_async else ''}def change_ul(self, ul: UserLevel, password: Optional[str] = None) -> int:\n"
+    result += "        assert isinstance(ul, UserLevel), f\"expected type 'UserLevel' for parameter 'ul', got '{type(ul)}'\"\n"
+    result += "        assert isinstance(password, str) or password is None, f\"expected type 'str' or 'None' for parameter 'password', got '{type(password)}'\"\n"
+    result += f"        return {'await ' if use_async else ''}self.__client.change_ul(ul, password)"
 
     return result
 
@@ -376,39 +439,42 @@ def generate_cmd(name: str, params: List[Tuple[str, str]], return_type: Optional
     _async = 'async ' if use_async else ''
     _await = 'await ' if use_async else ''
 
-    result = "    {_async}def {name}(self".format(_async=_async, name=make_name(name))
+    result = f"    {_async}def {make_function_name(name)}(self"
 
     if input_type is not None:
-        result += ", input_stream: {type}".format(type=make_stream_type(input_type))
+        result += f", input_stream: {make_stream_type(input_type)}"
 
     for param_name, param_type in params:
-        result += ", {name}: {type}".format(name=make_name(param_name), type=make_cmd_type(param_type))
+        result += f", {make_param_name(param_name)}: {make_cmd_type(param_type)}"
 
     if return_type is not None and output_type is not None:
-        result += ") -> Tuple[{stream}, {result}]:\n".format(stream=make_stream_type(output_type), result=make_cmd_type(return_type))
+        result += f") -> Tuple[{make_stream_type(output_type)}, {make_cmd_type(return_type)}]:\n"
     elif return_type is not None:
-        result += ") -> {result}:\n".format(result=make_cmd_type(return_type))
+        result += f") -> {make_cmd_type(return_type)}:\n"
     elif output_type is not None:
-        result += ") -> {stream}:\n".format(stream=make_stream_type(output_type))
+        result += f") -> {make_stream_type(output_type)}:\n"
     else:
         result += ") -> None:\n"
 
     if input_type is not None:
-        result += "        assert isinstance(input_stream, {type}), \"expected type '{type}' for parameter 'input_stream', got '{{}}'\".format(type(input_stream))\n".format(type=make_stream_type(input_type))
+        stream_type = make_stream_type(input_type)
+        result += f"        assert isinstance(input_stream, {stream_type}), f\"expected type '{stream_type}' for parameter 'input_stream', got '{{type(input_stream)}}'\"\n"
 
     for param_name, param_type in params:
-        result += "        assert isinstance({name}, {type}), \"expected type '{type}' for parameter '{name}', got '{{}}'\".format(type({name}))\n".format(name=make_name(param_name), type=make_cmd_type(param_type))
+        _name = make_param_name(param_name)
+        _type = make_cmd_type(param_type)
+        result += f"        assert isinstance({_name}, {_type}), f\"expected type '{_type}' for parameter '{_name}', got '{{type({_name})}}'\"\n"
 
     cmd_name = "self.__name + ':" + name + "'" if is_typedef_cmd else "'" + name + "'"
 
     if output_type is not None or return_type is not None:
-        result += "        return {_await}self.__client.exec({name}".format(_await=_await, name=cmd_name)
+        result += f"        return {_await}self.__client.exec({cmd_name}"
     else:
-        result += "        {_await}self.__client.exec({name}".format(_await=_await, name=cmd_name)
+        result += f"        {_await}self.__client.exec({cmd_name}"
 
     # Generate remaining parameters
     for param_name, _ in params:
-        result += ', ' + make_name(param_name)
+        result += ', ' + make_param_name(param_name)
 
     # Input stream parameter
     result += ", input_stream=input_stream" if input_type else ", input_stream=None"
@@ -570,21 +636,20 @@ def generate_atomic_typedef(typedef: Typedef, typenames: TypeNameMap, readlevel:
     # Getter
     type_list = [make_cmd_type(x.paramtype) for x in typedef.params.values()]
 
-    source += "    {}def get(self) -> Tuple[".format(_async) + ', '.join(type_list) + "]:\n"
-    source += "        return {}self.__client.get(self.__name)\n\n".format(_await)
+    source += f"    {_async}def get(self) -> Tuple[" + ', '.join(type_list) + "]:\n"
+    source += f"        return {_await}self.__client.get(self.__name)\n\n"
 
     # Setter
-    tmpl0 = "    {prefix}def set(self, {param_list}) -> None:\n"
-    tmpl1 = "        assert isinstance({x}, {type}), \"expected type '{type}' for '{x}', got '{{}}'\".format(type({x}))\n"
-    tmpl2 = "        {prefix}self.__client.set(self.__name, {param_names})\n\n"
-
-    param_list = [make_name(x.name) + ': ' + make_cmd_type(x.paramtype) for x in typedef.params.values()]
-    source += tmpl0.format(prefix=_async, param_list=', '.join(param_list))
+    param_list = [make_param_name(x.name) + ': ' + make_cmd_type(x.paramtype) for x in typedef.params.values()]
+    source += f"    {_async}def set(self, {', '.join(param_list)}) -> None:\n"
 
     for param in typedef.params.values():
-        source += tmpl1.format(x=make_name(param.name), type=make_cmd_type(param.paramtype))
+        x = make_param_name(param.name)
+        t = make_cmd_type(param.paramtype)
+        source += f"        assert isinstance({x}, {t}), f\"expected type '{t}' for '{x}', got '{{type({x})}}'\"\n"
 
-    source += tmpl2.format(prefix=_await, param_names=', '.join([make_name(x.name) for x in typedef.params.values()]))
+    set_params = ', '.join([make_param_name(x.name) for x in typedef.params.values()])
+    source += f"        {_await}self.__client.set(self.__name, {set_params})\n\n"
 
     # Commands
     if ul <= UserLevel.NORMAL:
@@ -604,7 +669,7 @@ def generate_atomic_typedef(typedef: Typedef, typenames: TypeNameMap, readlevel:
 
 def generate_typedefs_helper(model: SystemModel, typedef: Typedef, typenames: TypeNameMap, generated_types,
                              readlevel: UserLevel, writelevel: UserLevel, ul: UserLevel, use_async: bool) -> str:
-
+    """Helper function for generating typedefs."""
     result = ''
 
     for param in typedef.params.values():
@@ -743,58 +808,60 @@ def generate_header(model_name: str, use_async: bool, default_cmd_port: Optional
 
     """
     has_default_ports = default_cmd_port is not None or default_mon_port is not None
+    _asyncio = 'asyncio.' if use_async else ''
 
-    header = dedent("""\
-        # Generated from '{model}' on {date}
+    header = dedent(f"""\
+        # Generated from '{model_name}' on {str(datetime.now())}
 
         from typing import Tuple
+        from typing import Optional
 
-        from toptica.lasersdk.{_async}client import UserLevel
-        from toptica.lasersdk.{_async}client import Client
+        from toptica.lasersdk.{_asyncio}client import UserLevel
+        from toptica.lasersdk.{_asyncio}client import Client
 
-        from toptica.lasersdk.{_async}client import DecopError
-        from toptica.lasersdk.{_async}client import DeviceNotFoundError
+        from toptica.lasersdk.{_asyncio}client import DecopError
+        from toptica.lasersdk.{_asyncio}client import DeviceNotFoundError
 
-        from toptica.lasersdk.{_async}client import DecopBoolean
-        from toptica.lasersdk.{_async}client import DecopInteger
-        from toptica.lasersdk.{_async}client import DecopReal
-        from toptica.lasersdk.{_async}client import DecopString
-        from toptica.lasersdk.{_async}client import DecopBinary
+        from toptica.lasersdk.{_asyncio}client import DecopBoolean
+        from toptica.lasersdk.{_asyncio}client import DecopInteger
+        from toptica.lasersdk.{_asyncio}client import DecopReal
+        from toptica.lasersdk.{_asyncio}client import DecopString
+        from toptica.lasersdk.{_asyncio}client import DecopBinary
 
-        from toptica.lasersdk.{_async}client import MutableDecopBoolean
-        from toptica.lasersdk.{_async}client import MutableDecopInteger
-        from toptica.lasersdk.{_async}client import MutableDecopReal
-        from toptica.lasersdk.{_async}client import MutableDecopString
-        from toptica.lasersdk.{_async}client import MutableDecopBinary
+        from toptica.lasersdk.{_asyncio}client import MutableDecopBoolean
+        from toptica.lasersdk.{_asyncio}client import MutableDecopInteger
+        from toptica.lasersdk.{_asyncio}client import MutableDecopReal
+        from toptica.lasersdk.{_asyncio}client import MutableDecopString
+        from toptica.lasersdk.{_asyncio}client import MutableDecopBinary
 
-        from toptica.lasersdk.{_async}client import SettableDecopBoolean
-        from toptica.lasersdk.{_async}client import SettableDecopInteger
-        from toptica.lasersdk.{_async}client import SettableDecopReal
-        from toptica.lasersdk.{_async}client import SettableDecopString
-        from toptica.lasersdk.{_async}client import SettableDecopBinary
+        from toptica.lasersdk.{_asyncio}client import SettableDecopBoolean
+        from toptica.lasersdk.{_asyncio}client import SettableDecopInteger
+        from toptica.lasersdk.{_asyncio}client import SettableDecopReal
+        from toptica.lasersdk.{_asyncio}client import SettableDecopString
+        from toptica.lasersdk.{_asyncio}client import SettableDecopBinary
 
-        from toptica.lasersdk.{_async}client import Connection
-        from toptica.lasersdk.{_async}client import SerialConnection
-        """.format(model=model_name, date=str(datetime.now()), _async='asyncio.' if use_async else ''))
+        from toptica.lasersdk.{_asyncio}client import Connection
+        from toptica.lasersdk.{_asyncio}client import SerialConnection
+        """)
 
     if has_default_ports:
-        header += dedent("""\
+        header += dedent(f"""\
         
-            import toptica.lasersdk.{_async}client
+            import toptica.lasersdk.{'asyncio.' if use_async else ''}client
 
 
-            class NetworkConnection(toptica.lasersdk.{_async}client.NetworkConnection):
-                def __init__(self, host: str, command_line_port: int = {cmd_port}, monitoring_line_port: int = {mon_port}, timeout: int = 5) -> None:
+            class NetworkConnection(toptica.lasersdk.{'asyncio.' if use_async else ''}client.NetworkConnection):
+                def __init__(self, host: str, command_line_port: int = {default_cmd_port or 0}, monitoring_line_port: int = {default_mon_port or 0}, timeout: int = 5) -> None:
                     super().__init__(host, command_line_port, monitoring_line_port, timeout)
 
 
-            """.format(_async='asyncio.' if use_async else '', cmd_port=default_cmd_port or 0, mon_port=default_mon_port or 0))
+            """)
     else:
-        header += dedent("""\
-            from toptica.lasersdk.{_async}client import NetworkConnection
+        header += dedent(f"""\
+            from toptica.lasersdk.{'asyncio.' if use_async else ''}client import NetworkConnection
 
 
-            """.format(_async='asyncio.' if use_async else ''))
+            """)
 
     return header
 
